/**
  *  \file client/widgets/fleetlist.h
  *  \brief Fleet List
  *
  *  (c) 2001-2019 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CLIENT_WIDGETS_FLEETLIST_H
#define PCC_V2_CLIENT_WIDGETS_FLEETLIST_H

#include <vector>
#include "cpluslib/string.h"
#include "game/coord.h"
#include "ui/listbox.h"

class GUniverse;

/** List of fleets. This generates and stores a list of fleets. */
class WFleetList {
 public:
    enum {
        fAllFleets = 1,         ///< List all fleets (not just those at the position).
        fQuickOut  = 2,         ///< No dialog if only one choice.
    };

    // Construction
    WFleetList();
    ~WFleetList();

    // Generation
    void listFleets(const GUniverse& univ, GPoint pt, int flags, int except);

    // Access
    uint16_t operator[](std::size_t index) const;
    bool isHere(std::size_t index) const;
    std::size_t size() const;
    std::size_t getInitialSelection() const;

 private:
    enum {
        mHere = 0x8000
    };

    std::vector<uint16_t> fleet_ids;
    std::size_t initial_selection;
};

/** Fleet list widget. This displays a WFleetList. */
class WFleetListWidget : public UIListbox {
    const WFleetList& list;
 public:
    WFleetListWidget(const WFleetList& list, int nlines);
    ~WFleetListWidget();
    void drawPart(GfxCanvas& can, int from, int to);
};

int doFleetList(const string_t title, GUniverse& univ, GPoint pt, int flags, int except);



/***************************** Inline Methods ****************************/

/** Get fleet member.
    \param index Index [0,size())
    \return fleet Id (=ship Id of leader) */
inline uint16_t
WFleetList::operator[](std::size_t index) const
{
    return fleet_ids[index] & ~mHere;
}

/** Check whether fleet is "here".
    \param index Index [0,size()) */
inline bool
WFleetList::isHere(std::size_t index) const
{
    return (fleet_ids[index] & mHere) != 0;
}

/** Get number of fleets in list. */
inline std::size_t
WFleetList::size() const
{
    return fleet_ids.size();
}

/** Get initial selection. This is the first fleet which is "here", if any. */
inline std::size_t
WFleetList::getInitialSelection() const
{
    return initial_selection;
}

#endif
