/**
  *  \file game/bum.h
  *  \brief Beam up Multiple
  *
  *  (c) 2001-2019 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_BUM_H
#define PCC_V2_GAME_BUM_H

#include "game/cargo.h"

class GShip;
class GPlanet;
class GGameTurn;

/** Beam up Multiple Transfer: Ship part. This has the amounts in the
    have part, and the actual current ship content in display_offsets. 
    Upon commit, it generates the command and mission. */
class GShipBumTransfer : public GCargoContainer {
 public:
    GShipBumTransfer(GGameTurn& trn, int sid);
    ~GShipBumTransfer();

    bool canHaveCargo(GCargoType cargo_id);
    int32_t getMaxCargo(GCargoType cargo_id);
    int32_t getDisplayOffset(GCargoType cargo_id);
    string_t getName() const;

 protected:
    int32_t getCargoOnObject(GCargoType type);
    void changeCargoOnObject(GCargoType type, int32_t amount);
    void changeReservedCargo(GCargoType type, int32_t amount);
    void commit();

 private:
    void init();
    GGameTurn& trn;
    int sid;
    int32_t have[el_COUNT];
    int32_t display_offsets[el_COUNT];
    int32_t reserved[el_COUNT];
};

/** Beam up Multiple Transfer: Planet part. This offers 10000 (minus
    the amounts already beamed up) of everything, and adjusts that
    back to the actual planet's content (or zero) using
    display_offsets. It is otherwise completely passive, as
    mission/command generation is taken over by the ship part. */
class GPlanetBumTransfer : public GCargoContainer {
 public:
    GPlanetBumTransfer(GGameTurn& trn, GPlanet& pl, int sid);
    ~GPlanetBumTransfer();

    bool canHaveCargo(GCargoType cargo_id);
    int32_t getMaxCargo(GCargoType cargo_id);
    int32_t getDisplayOffset(GCargoType cargo_id);
    string_t getName() const;

 protected:
    int32_t getCargoOnObject(GCargoType type);
    void changeCargoOnObject(GCargoType type, int32_t amount);
    void changeReservedCargo(GCargoType type, int32_t amount);

 private:
    void init(int sid);
    GGameTurn& trn;
    GPlanet& pl;
    int32_t have[el_COUNT];
    int32_t display_offsets[el_COUNT];
    int32_t reserved[el_COUNT];
};

#endif
