/**
  *  \file io/sdlstr.h
  *  \brief SDL<->Stream interface
  *
  *  (c) 2001-2019 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_IO_SDLSTR_H
#define PCC_V2_IO_SDLSTR_H

#include "defs.h"

#if HAVE_SDL
#  include <SDL_rwops.h>
#  include "io/streams.h"

extern "C" {
    int SDLIF_seek(SDL_RWops* context, int offset, int whence);
    int SDLIF_read(SDL_RWops* context, void* ptr, int size, int maxnum);
    int SDLIF_write(SDL_RWops* context, const void* ptr, int size, int num);
    int SDLIF_close(SDL_RWops*);
}

/*! \class StreamSDLInterface
    \brief Stream Interface to SDL_RWops

    This class translates a Stream object into the SDL_RWops interface
    for SDL routines, such as SDL_LoadBMP(). Lifetime management is still
    up to the caller, the close() function exported to SDL is a no-op.

    (This used to be part of the Stream class itself, requiring all
    programs including textual ones to be linked to SDL.) */
class StreamSDLInterface : public SDL_RWops {
    Stream& s;
    friend int SDLIF_seek(SDL_RWops* context, int offset, int whence);
    friend int SDLIF_read(SDL_RWops* context, void* ptr, int size, int maxnum);
    friend int SDLIF_write(SDL_RWops* context, const void* ptr, int size, int num);
    friend int SDLIF_close(SDL_RWops*);
 public:
    StreamSDLInterface(Stream& s);
};

#endif /* HAVE_SDL */

#endif
