/**
  *  \file util/modinit.h
  *  \brief Module Initialisation
  *
  *  (c) 2001-2019 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_UTIL_MODINIT_H
#define PCC_V2_UTIL_MODINIT_H

#include "cpluslib/types.h"

/* Note: scripts can be run first after NoGame. */
enum {
    OnStartup        = 1,    /* We are starting. No game access yet. Next is Shutdown, NoGame, or Directory. */
    OnShutdown       = 2,    /* We are closing down. */
    OnEnterNoGame    = 4,    /* We enter a no-game state. No game data, spec dirs point to root. Next is Directory or Shutdown. */
    OnEnterDirectory = 8,    /* We enter a directory state. No game data, spec dirs point to a game. Next is LeftDirectory or EnterGame. */
    OnEnterGame      = 16,   /* We enter a game. No game data yet, spec dirs point to a game. Next is EnteredGame. CCScript: "On BeforeLoad". */
    OnEnteredGame    = 32,   /* We entered a game. Game data fully loaded. Next is LeaveGame. CCScript: "On Load". */
    OnLeaveGame      = 64,   /* We are leaving the game. Game data still fully loaded. CCScript: "On Exit". */
    OnLeftGame       = 128,  /* We have left the game. Game data no longer present, spec dirs point to game. Next is Directory or LeftDirectory. CCScript: "On AfterExit". */
    OnLeftDirectory  = 256,  /* We have left a game. Spec dirs still point to game. Next is NoGame or Shutdown. */
};

struct ModuleInitializer {
    uint32_t condition;
    void (*initializer)();
};

void performInitialisation(uint32_t flags, const ModuleInitializer* init);
void performInitialisation(uint32_t flags, const ModuleInitializer*const* init);

extern const ModuleInitializer mod_base_gui[];
extern const ModuleInitializer mod_base_console[];

#endif
